@@ -19,9 +19,9 @@
 
 """Provides support for synthesizing keyboard and mouse events."""
 
-__id__        = "$Id: eventsynthesizer.py 3153 2007-10-22 19:18:04Z eitani $"
-__version__   = "$Revision: 3153 $"
-__date__      = "$Date: 2007-10-22 15:18:04 -0400 (Mon, 22 Oct 2007) $"
+__id__        = "$Id: eventsynthesizer.py 3290 2007-11-29 22:09:59Z richb $"
+__version__   = "$Revision: 3290 $"
+__date__      = "$Date: 2007-11-29 17:09:59 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -72,7 +72,7 @@ def clickPoint(x, y, button):
 
     generateMouseEvent(x, y, "b%dc" % button)
 
-def generateKeyboardEvent(keycode, keystring, type):
+def generateKeyboardEvent(keycode, keystring, eventType):
     """Generates a keyboard event.
 
     Arguments:
@@ -83,10 +83,10 @@ def generateKeyboardEvent(keycode, keystring, type):
               being synthesized; this type of keyboard event synthesis does
               not emulate hardware keypresses but injects the string
               as though a composing input method (such as XIM) were used.
-    - type:   an AccessibleKeySynthType flag indicating whether keyval
+    - eventType: an AccessibleKeySynthType flag indicating whether keyval
               is to be interpreted as a keysym rather than a keycode
               (pyatspi.KEY_SYM), or whether to synthesize
               KEY_PRESS, KEY_RELEASE, or both (KEY_PRESSRELEASE).
     """
 
-    pyatspi.Registry.generateKeyboardEvent(keycode, keystring, type)
+    pyatspi.Registry.generateKeyboardEvent(keycode, keystring, eventType)
