@@ -20,9 +20,9 @@
 """Provides support for defining keybindings and matching them to input
 events."""
 
-__id__        = "$Id: keybindings.py 2351 2007-05-07 21:56:41Z wwalker $"
-__version__   = "$Revision: 2351 $"
-__date__      = "$Date: 2007-05-07 17:56:41 -0400 (Mon, 07 May 2007) $"
+__id__        = "$Id: keybindings.py 3561 2008-02-11 18:11:45Z wwalker $"
+__version__   = "$Revision: 3561 $"
+__date__      = "$Date: 2008-02-11 13:11:45 -0500 (Mon, 11 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -33,12 +33,13 @@ try:
     # the desktop is not running.
     #
     import gtk
-except:
+except ImportError:
     pass
 
-import atspi
+import pyatspi
 import debug
 import settings
+import orca_state
 
 from orca_i18n import _           # for gettext support
 
@@ -114,6 +115,10 @@ def getKeycode(keysym):
     Returns an integer representing a key code that should match the
     event.hw_code for key events.
     """
+
+    if not keysym:
+        return 0
+
     if not _keycodeCache.has_key(keysym):
         keymap = gtk.gdk.keymap_get_default()
 
@@ -150,41 +155,41 @@ def getModifierNames(mods):
     text = ""
     if mods & (1 << settings.MODIFIER_ORCA):
         text += _("Orca") + "+"
-    #if mods & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
+    #if mods & (1 << pyatspi.MODIFIER_NUMLOCK):
     #    text += _("Num_Lock") + "+"
     if mods & 128:
         # Translators: this is presented in a GUI to represent the
         # "right alt" modifier.
         #
         text += _("Alt_R") + "+"
-    if mods & (1 << atspi.Accessibility.MODIFIER_META3):
+    if mods & (1 << pyatspi.MODIFIER_META3):
         # Translators: this is presented in a GUI to represent the
         # "super" modifier.
         #
         text += _("Super") + "+"
-    if mods & (1 << atspi.Accessibility.MODIFIER_META2):
+    if mods & (1 << pyatspi.MODIFIER_META2):
         # Translators: this is presented in a GUI to represent the
         # "meta 2" modifier.
         #
         text += _("Meta2") + "+"
-    #if mods & (1 << atspi.Accessibility.MODIFIER_META):
+    #if mods & (1 << pyatspi.MODIFIER_META):
     #    text += _("Meta") + "+"
-    if mods & (1 << atspi.Accessibility.MODIFIER_ALT):
+    if mods & (1 << pyatspi.MODIFIER_ALT):
         # Translators: this is presented in a GUI to represent the
         # "left alt" modifier.
         #
         text += _("Alt_L") + "+"
-    if mods & (1 << atspi.Accessibility.MODIFIER_CONTROL):
+    if mods & (1 << pyatspi.MODIFIER_CONTROL):
         # Translators: this is presented in a GUI to represent the
         # "control" modifier.
         #
         text += _("Ctrl") + "+"
-    if mods & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
+    if mods & (1 << pyatspi.MODIFIER_SHIFTLOCK):
         # Translators: this is presented in a GUI to represent the
         # "caps lock" modifier.
         #
         text += _("Caps_Lock") + "+"
-    if mods & (1 << atspi.Accessibility.MODIFIER_SHIFT):
+    if mods & (1 << pyatspi.MODIFIER_SHIFT):
         # Translators: this is presented in a GUI to represent the
         # "shift " modifier.
         #
@@ -196,7 +201,8 @@ class KeyBinding:
     and the InputEventHandler.
     """
 
-    def __init__(self, keysymstring, modifier_mask, modifiers, handler):
+    def __init__(self, keysymstring, modifier_mask, modifiers, handler,
+                 click_count = 1):
         """Creates a new key binding.
 
         Arguments:
@@ -204,10 +210,10 @@ class KeyBinding:
           from /usr/include/X11/keysymdef.h with the preceding 'XK_'
           removed (e.g., XK_KP_Enter becomes the string 'KP_Enter').
         - modifier_mask: bit mask where a set bit tells us what modifiers
-          we care about (see atspi.Accessibility.MODIFIER_*)
+          we care about (see pyatspi.MODIFIER_*)
         - modifiers: the state the modifiers we care about must be in for
           this key binding to match an input event (see also
-          atspi.Accessibility.MODIFIER_*)
+          pyatspi.MODIFIER_*)
         - handler: the InputEventHandler for this key binding
         """
 
@@ -215,6 +221,7 @@ class KeyBinding:
         self.modifier_mask = modifier_mask
         self.modifiers = modifiers
         self.handler = handler
+        self.click_count = click_count
         self.keycode = None
 
     def matches(self, keycode, modifiers):
@@ -245,10 +252,12 @@ class KeyBindings:
     def __str__(self):
         result = "[\n"
         for keyBinding in self.keyBindings:
-            result += "  [%x %x %s %s]\n" % (keyBinding.modifier_mask,
-                                             keyBinding.modifiers,
-                                             keyBinding.keysymstring,
-                                             keyBinding.handler._description)
+            result += "  [%x %x %s %d %s]\n" % \
+                      (keyBinding.modifier_mask,
+                       keyBinding.modifiers,
+                       keyBinding.keysymstring,
+                       keyBinding.click_count,
+                       keyBinding.handler.description)
         result += "]"
         return result
     
@@ -279,27 +288,32 @@ class KeyBindings:
         """Return True if keyBinding is already in self.keyBindings.
 
            The typeOfSearch can be:
-              "strict":      matches description, modifiers, key
+              "strict":      matches description, modifiers, key, and
+                             click count
               "description": matches only description.
-              "keys":        matches only modifiers and key.
+              "keys":        matches the modifiers, key, and modifier mask,
+                             and click count
+              "keysNoMask":  matches the modifiers, key, and click count
         """
 
         hasIt = False
 
         for keyBinding in self.keyBindings:
             if typeOfSearch == "strict":
-                if (keyBinding.handler._description \
-                    == newKeyBinding.handler._description) \
+                if (keyBinding.handler.description \
+                    == newKeyBinding.handler.description) \
                     and (keyBinding.keysymstring \
                          == newKeyBinding.keysymstring) \
                     and (keyBinding.modifier_mask \
                          == newKeyBinding.modifier_mask) \
                     and (keyBinding.modifiers \
-                         == newKeyBinding.modifiers):
+                         == newKeyBinding.modifiers) \
+                    and (keyBinding.click_count \
+                         == newKeyBinding.click_count):
                     hasIt = True
             elif typeOfSearch == "description":
-                if keyBinding.handler._description \
-                    == newKeyBinding.handler._description:
+                if keyBinding.handler.description \
+                    == newKeyBinding.handler.description:
                     hasIt = True
             elif typeOfSearch == "keys":
                 if (keyBinding.keysymstring \
@@ -307,7 +321,17 @@ class KeyBindings:
                     and (keyBinding.modifier_mask \
                          == newKeyBinding.modifier_mask) \
                     and (keyBinding.modifiers \
-                         == newKeyBinding.modifiers):
+                         == newKeyBinding.modifiers) \
+                    and (keyBinding.click_count \
+                         == newKeyBinding.click_count):
+                    hasIt = True
+            elif typeOfSearch == "keysNoMask":
+                if (keyBinding.keysymstring \
+                    == newKeyBinding.keysymstring) \
+                    and (keyBinding.modifiers \
+                         == newKeyBinding.modifiers) \
+                    and (keyBinding.click_count \
+                         == newKeyBinding.click_count):
                     hasIt = True
 
         return hasIt
@@ -316,13 +340,31 @@ class KeyBindings:
         """Returns the input handler of the key binding that matches the
         given keycode and modifiers, or None if no match exists.
         """
-        handler = None
+
+        if not orca_state.activeScript:
+            return None
+
+        candidates = []
+        clickCount = orca_state.activeScript.getClickCount()
         for keyBinding in self.keyBindings:
             if keyBinding.matches(keyboardEvent.hw_code, \
                                   keyboardEvent.modifiers):
-                handler = keyBinding.handler
-                break
-        return handler
+                if keyBinding.modifier_mask == keyboardEvent.modifiers and \
+                   keyBinding.click_count == clickCount:
+                    return keyBinding.handler
+                candidates.append(keyBinding)
+
+        # If we're still here, we don't have an exact match. Prefer
+        # the one whose click count is closest to, but does not exceed,
+        # the actual click count.
+        #
+        candidates.sort(cmp=lambda x, y: x.click_count - y.click_count,
+                        reverse=True)
+        for candidate in candidates:
+            if candidate.click_count <= clickCount:
+                return candidate.handler
+
+        return None
 
     def consumeKeyboardEvent(self, script, keyboardEvent):
         """Attempts to consume the given keyboard event.  If these
@@ -334,7 +376,7 @@ class KeyBindings:
         handler = self.getInputHandler(keyboardEvent)
         if handler:
             consumed = True
-            if keyboardEvent.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+            if keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
                 try:
                     handler.processInputEvent(script, keyboardEvent)
                 except:
