@@ -19,9 +19,9 @@
 
 """Displays a GUI for the user to quit Orca."""
 
-__id__        = "$Id: orca_glade.py 3077 2007-10-17 16:32:55Z richb $"
-__version__   = "$Revision: 3077 $"
-__date__      = "$Date: 2007-10-17 12:32:55 -0400 (Wed, 17 Oct 2007) $"
+__id__        = "$Id: orca_glade.py 3288 2007-11-29 21:51:00Z richb $"
+__version__   = "$Revision: 3288 $"
+__date__      = "$Date: 2007-11-29 16:51:00 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -40,10 +40,10 @@ class GladeWrapper:
         subclass_instance.widget_name...
     """
 
-    def __init__(self, Filename, WindowName):
+    def __init__(self, fileName, windowName):
         # Load glade file.
-        self.widgets = gtk.glade.XML(Filename, WindowName, gettext.textdomain())
-        self.GtkWindow = getattr(self, WindowName)
+        self.widgets = gtk.glade.XML(fileName, windowName, gettext.textdomain())
+        self.gtkWindow = getattr(self, windowName)
 
         instance_attributes = {}
         for attribute in dir(self.__class__):
