@@ -21,9 +21,9 @@
 of manipulating text - both the user and the system put text into the
 system and we try to determine which was which and why."""
 
-__id__        = "$Id: gnome-terminal.py 3295 2007-11-29 23:08:59Z richb $"
-__version__   = "$Revision: 3295 $"
-__date__      = "$Date: 2007-11-29 18:08:59 -0500 (Thu, 29 Nov 2007) $"
+__id__        = "$Id: gnome-terminal.py 3496 2008-01-24 17:40:12Z richb $"
+__version__   = "$Revision: 3496 $"
+__date__      = "$Date: 2008-01-24 12:40:12 -0500 (Thu, 24 Jan 2008) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -31,6 +31,7 @@ import pyatspi
 
 import orca.default as default
 import orca.input_event as input_event
+import orca.orca as orca
 import orca.orca_state as orca_state
 import orca.settings as settings
 import orca.speech as speech
@@ -57,6 +58,26 @@ class Script(default.Script):
         #
         self.presentIfInactive = False
 
+    def echoKey(self, keyEvent):
+        """Determine whether this script should echo the current key event.
+        If this is a printable key, then return False.
+
+        Note that the keyEcho() method in orca.py will still take into
+        account whatever the user's various preferences for key echoing
+        are, which may override what is return by this echoKey() method.
+
+        Arguments:
+        - keyEvent - the key event
+
+        Returns an indication of whether a key echo event should be
+        allowed to happen for this script.
+        """
+
+        if orca.isPrintableKey(keyEvent.event_string):
+            return False
+
+        return True
+
     #def onWindowActivated(self, event):
     #    # Sets the context to the top level window first, so we can
     #    # get information about it the window we just moved to.
@@ -163,7 +184,8 @@ class Script(default.Script):
 
             # If the last input event was a keyboard event, check to see if
             # the text for this event matches what the user typed. If it does,
-            # then don't speak it.
+            # then call orca.keyEcho() to echo it (based on the user's key
+            # echo preferences).
             #
             # Note that the text widgets sometimes compress their events,
             # thus we might get a longer string from a single text inserted
@@ -204,6 +226,9 @@ class Script(default.Script):
              orca_state.lastInputEvent.button == "2":
             speakThis = True
 
+        if matchFound:
+            orca.keyEcho(orca_state.lastInputEvent)
+
         if speakThis:
             if text.isupper():
                 speech.speak(text, self.voices[settings.UPPERCASE_VOICE])
